unit FormUnit1;

interface

uses
  Bcl.JOSE.Core.Base,
  Bcl.JOSE.Core.JWT,
  Bcl.JOSE.Core.JWS,
  Bcl.JOSE.Core.JWK,
  Bcl.JOSE.Core.JWA,
  Bcl.JOSE.Types.JSON,
  Bcl.JOSE.Core.Builder,
  Bcl.JOSE.Consumer,
  Bcl.JOSE.Context,
  IdHTTPWebBrokerBridge,
  IdGlobal,
  IdContext,
  IdCustomHttpServer,
  System.SysUtils,
  System.Variants,
  System.Classes,
  System.JSON,
  Vcl.Graphics,
  Vcl.Controls,
  Vcl.Forms,
  Vcl.Dialogs,
  Vcl.AppEvnts,
  Vcl.StdCtrls,
  Web.HTTPApp,
  Winapi.Messages,
  Winapi.Windows,
  Winapi.ShellApi;

const
  SHARED_SECRET = 'qwertyuiopasdfghjklzxcvbnm123456';

type
  TForm1 = class(TForm)
    ButtonStart: TButton;
    ButtonStop: TButton;
    EditPort: TEdit;
    Label1: TLabel;
    ApplicationEvents1: TApplicationEvents;
    ButtonOpenBrowser: TButton;
    Memo1: TMemo;
    Label2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
    procedure ButtonStartClick(Sender: TObject);
    procedure ButtonStopClick(Sender: TObject);
    procedure ButtonOpenBrowserClick(Sender: TObject);
  private
    FServer: TIdHTTPWebBrokerBridge;
    function GetFieldValueFromJSON(AJSONString: String; AFieldName: String): String;
    procedure StartServer;
    procedure DoParseAuthentication(AContext: TIdContext; const AAuthType, AAuthData: String;
      var VUsername, VPassword: String; var VHandled: Boolean);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses
  GraphQL.Bookshelf;

{$R *.dfm}

procedure TForm1.ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
begin
  ButtonStart.Enabled := not FServer.Active;
  ButtonStop.Enabled := FServer.Active;
  EditPort.Enabled := not FServer.Active;
end;

procedure TForm1.ButtonOpenBrowserClick(Sender: TObject);
var
  LURL: string;
begin
  StartServer;
  LURL := Format('http://localhost:%s/graphql/', [EditPort.Text]);
  ShellExecute(0,
        nil,
        PChar(LURL), nil, nil, SW_SHOWNOACTIVATE);
end;

procedure TForm1.ButtonStartClick(Sender: TObject);
begin
  StartServer;
end;

procedure TForm1.ButtonStopClick(Sender: TObject);
begin
  FServer.Active := False;
  FServer.Bindings.Clear;
end;

procedure TForm1.DoParseAuthentication(AContext: TIdContext; const AAuthType, AAuthData: String; var VUsername, VPassword: String;
  var VHandled: Boolean);
var
  LToken: string;
  LJWT: TJWT;
  LBuilder: IJOSEConsumerBuilder;
  LConsumer: IJOSEConsumer;
  LCustomerEmail: String;
begin
  VHandled := False;
  if SameText(AAuthType, 'Bearer') and (AAuthData <> '') then
  begin
    LToken := AAuthData;
    try
      LJWT := TJOSE.DeserializeOnly(LToken);
      try
        // Build consumer
        LBuilder := TJOSEConsumerBuilder.NewConsumer
          .SetVerificationKey(SHARED_SECRET)
          .SetExpectedAlgorithms([TJOSEAlgorithmId.HS256]);

        // Recommended to set expected audience and issuer
        LBuilder.SetSkipDefaultAudienceValidation;
//        LBuilder.SetExpectedAudience(True, 'http://my-api-url');
//        LBuilder.SetExpectedIssuers(True, 'http://url-of-jwt-issuer');

        LConsumer := LBuilder.Build;
        LConsumer.Process(LToken);

        // Getting the value of customer_email
        LCustomerEmail := GetFieldValueFromJSON(LJWT.Claims.JSON.ToString, 'customer_email');
        GraphQL.Bookshelf.CustomerEmailFromJWT := LCustomerEmail;
        OutputDebugString(PChar('customer_email: ' + LCustomerEmail)); // just for debugging; remove this line later
        // --end

        VHandled := True;
      finally
        LJWT.Free;
      end;
    except
      on E: EJOSEException do
        raise EIdHTTPUnsupportedAuthorisationScheme.Create(E.Message);
      on E: EInvalidJWTException do
        raise EIdHTTPUnsupportedAuthorisationScheme.Create(E.Message);
      else
        raise;
    end;
  end;
end;

function TForm1.GetFieldValueFromJSON(AJSONString: String; AFieldName: String): String;
var
  LJSONObject: TJSONObject;
begin
  LJSONObject := TJSONObject.ParseJSONValue(AJSONString) as TJSONObject;
  try
    Result := LJSONObject.GetValue(AFieldName).Value;
  finally
    LJSONObject.Free;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FServer := TIdHTTPWebBrokerBridge.Create(Self);
  FServer.OnParseAuthentication := DoParseAuthentication;
end;

procedure TForm1.StartServer;
begin
  if not FServer.Active then
  begin
    FServer.Bindings.Clear;
    FServer.DefaultPort := StrToInt(EditPort.Text);
    FServer.Active := True;
  end;
end;

end.
